@@ -1,8 +1,7 @@
 {-# LANGUAGE OverloadedStrings #-}
 module WaiExtraTest (specs) where
 
-import Test.Hspec.Monadic
-import Test.Hspec.HUnit ()
+import Test.Hspec
 import Test.HUnit hiding (Test)
 
 import Network.Wai
@@ -13,6 +12,7 @@ import qualified Data.ByteString.Char8 as S8
 import qualified Data.ByteString.Lazy.Char8 as L8
 import qualified Data.ByteString.Lazy as L
 import qualified Data.Text.Lazy as T
+import qualified Data.Text as TS
 import qualified Data.Text.Encoding as TE
 import Control.Arrow
 
@@ -23,7 +23,7 @@ import Network.Wai.Middleware.Autohead
 import Network.Wai.Middleware.MethodOverride
 import Network.Wai.Middleware.MethodOverridePost
 import Network.Wai.Middleware.AcceptOverride
-import Network.Wai.Middleware.RequestLogger (logCallback)
+import Network.Wai.Middleware.RequestLogger
 import Codec.Compression.GZip (decompress)
 
 import qualified Data.Conduit as C
@@ -32,14 +32,27 @@ import Data.Conduit.Binary (sourceFile)
 import Control.Monad.IO.Class (liftIO)
 import Data.Maybe (fromMaybe)
 import Network.HTTP.Types (parseSimpleQuery, status200)
+import System.Log.FastLogger
 
-specs :: Specs
+import qualified Data.IORef as I
+
+specs :: Spec
 specs = do
   describe "Network.Wai.Parse" $ do
+    describe "parseContentType" $ do
+        let go (x, y, z) = it (TS.unpack $ TE.decodeUtf8 x) $ parseContentType x `shouldBe` (y, z)
+        mapM_ go
+            [ ("text/plain", "text/plain", [])
+            , ("text/plain; charset=UTF-8 ", "text/plain", [("charset", "UTF-8")])
+            , ("text/plain; charset=UTF-8 ; boundary = foo", "text/plain", [("charset", "UTF-8"), ("boundary", "foo")])
+            ]
     it "parseQueryString" caseParseQueryString
     it "parseQueryString with question mark" caseParseQueryStringQM
     it "parseHttpAccept" caseParseHttpAccept
     it "parseRequestBody" caseParseRequestBody
+    it "multipart with plus" caseMultipartPlus
+    it "multipart with multiple attributes" caseMultipartAttrs
+    it "urlencoded with plus" caseUrlEncPlus
     {-
     , it "findBound" caseFindBound
     , it "sinkTillBound" caseSinkTillBound
@@ -55,7 +68,9 @@ specs = do
     it "method override" caseMethodOverride
     it "method override post" caseMethodOverridePost
     it "accept override" caseAcceptOverride
-    it "dalvik multipart" caseDalvikMultipart
+    describe "dalvik multipart" $ do
+        it "non-chunked" $ dalvikHelper True
+        it "chunked" $ dalvikHelper False
     it "debug request body" caseDebugRequestBody
 
 caseParseQueryString :: Assertion
@@ -93,8 +108,8 @@ caseParseQueryStringQM = do
 
 caseParseHttpAccept :: Assertion
 caseParseHttpAccept = do
-    let input = "text/plain; q=0.5, text/html, text/x-dvi; q=0.8, text/x-c"
-        expected = ["text/html", "text/x-c", "text/x-dvi", "text/plain"]
+    let input = "text/plain; q=0.5, text/html;charset=utf-8, text/*;q=0.8;ext=blah, text/x-dvi; q=0.8, text/x-c"
+        expected = ["text/html;charset=utf-8", "text/x-c", "text/x-dvi", "text/*", "text/plain"]
     expected @=? parseHttpAccept input
 
 parseRequestBody' :: BackEnd L.ByteString
@@ -169,6 +184,40 @@ caseParseRequestBody =
                     expected3
                     result3'
 
+caseMultipartPlus :: Assertion
+caseMultipartPlus = do
+    result <- C.runResourceT $ parseRequestBody' lbsBackEnd $ toRequest ctype content
+    liftIO $ result @?= ([("email", "has+plus")], [])
+  where
+    content = S8.pack $
+        "--AaB03x\n" ++
+        "Content-Disposition: form-data; name=\"email\"\n" ++
+        "Content-Type: text/plain; charset=iso-8859-1\n\n" ++
+        "has+plus\n" ++
+        "--AaB03x--"
+    ctype = "multipart/form-data; boundary=AaB03x"
+
+caseMultipartAttrs :: Assertion
+caseMultipartAttrs = do
+    result <- C.runResourceT $ parseRequestBody' lbsBackEnd $ toRequest ctype content
+    liftIO $ result @?= ([("email", "has+plus")], [])
+  where
+    content = S8.pack $
+        "--AaB03x\n" ++
+        "Content-Disposition: form-data; name=\"email\"\n" ++
+        "Content-Type: text/plain; charset=iso-8859-1\n\n" ++
+        "has+plus\n" ++
+        "--AaB03x--"
+    ctype = "multipart/form-data; charset=UTF-8; boundary=AaB03x"
+
+caseUrlEncPlus :: Assertion
+caseUrlEncPlus = do
+    result <- C.runResourceT $ parseRequestBody' lbsBackEnd $ toRequest ctype content
+    liftIO $ result @?= ([("email", "has+plus")], [])
+  where
+    content = S8.pack $ "email=has%2Bplus"
+    ctype = "application/x-www-form-urlencoded"
+
 toRequest :: S8.ByteString -> S8.ByteString -> SRequest
 toRequest ctype content = SRequest defaultRequest
     { requestHeaders = [("Content-Type", ctype)]
@@ -405,11 +454,10 @@ caseAcceptOverride = flip runSession aoApp $ do
                 }
     assertHeader "Accept" "baz" sres3
 
-caseDalvikMultipart :: Assertion
-caseDalvikMultipart = do
-    let headers =
-            [ ("content-length", "12098")
-            , ("content-type", "multipart/form-data;boundary=*****")
+dalvikHelper :: Bool -> Assertion
+dalvikHelper includeLength = do
+    let headers' =
+            [ ("content-type", "multipart/form-data;boundary=*****")
             , ("GATEWAY_INTERFACE", "CGI/1.1")
             , ("PATH_INFO", "/")
             , ("QUERY_STRING", "")
@@ -425,6 +473,9 @@ caseDalvikMultipart = do
             , ("HTTP_VERSION", "HTTP/1.1")
             , ("REQUEST_PATH", "/")
             ]
+        headers
+            | includeLength = ("content-length", "12098") : headers'
+            | otherwise = headers'
     let request' = defaultRequest
             { requestHeaders = headers
             }
@@ -446,7 +497,7 @@ caseDebugRequestBody = do
         assertStatus 200 res
 
     let qs = "?foo=bar&baz=bin"
-    flip runSession (debugApp $ getOutput qs) $ do
+    flip runSession (debugApp $ getOutput params) $ do
         assertStatus 200 =<< request defaultRequest
                 { requestMethod = "GET"
                 , queryString = map (\(k,v) -> (k, Just v)) params
@@ -457,13 +508,25 @@ caseDebugRequestBody = do
   where
     params = [("foo", "bar"), ("baz", "bin")]
     -- FIXME change back once we include post parameter output in logging postOutput = T.pack $ "POST \nAccept: \nPOST " ++ (show params)
-    postOutput = T.pack $ "POST / Accept: \nStatus: 200 OK"
-    -- FIXME getOutput _qs = T.pack $ "GET /location" ++ "\nAccept: \nGET " ++ (show params) -- \nAccept: \n" ++ (show params)
-    getOutput _qs = T.pack $ "GET /location?foo=bar&baz=bin Accept: \nStatus: 200 OK"
+    postOutput = T.pack $ "POST /\nAccept: \nStatus: 200 OK. /\n"
+    getOutput params' = T.pack $ "GET /location\nAccept: \nGET " ++ show params' ++ "\nStatus: 200 OK. /location\n"
 
-    debugApp output' = logCallback (\t -> liftIO $ assertEqual "debug" output t) $ \_req -> do
-        return $ responseLBS status200 [ ] ""
+    debugApp output' req = do
+        iactual <- liftIO $ I.newIORef []
+        middleware <- liftIO $ mkRequestLogger def
+            { destination = Callback $ \strs -> I.modifyIORef iactual $ (++ strs)
+            , outputFormat = Detailed False
+            }
+        res <- middleware (\_req -> return $ responseLBS status200 [ ] "") req
+        actual <- liftIO $ I.readIORef iactual
+        liftIO $ assertEqual "debug" output $ logsToBs actual
+        return res
       where
         output = TE.encodeUtf8 $ T.toStrict output'
+        logsToBs = S.concat . map logToBs
+
+        logToBs (LB bs) = bs
+        logToBs (LS s) = S8.pack s
+
     {-debugApp = debug $ \req -> do-}
         {-return $ responseLBS status200 [ ] ""-}
